#include <stream.h>
#include <iostream.h>
#include <fstream.h>
#include <iomanip.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
//#include <time.h>
//#define pi 3.14159265358979323844

#define AMAX_ARRAY 26       //This is the max number of part.s or holes
#define NLEVELS_ARRAY 130   //This is the max number of single-particle levels
#define EMAX_ARRAY 200      //This sets the max energy = EMAX_ARRAY * ESCALE
#define JMAX_ARRAY 93       //This is the largest J to consider
#define ESCALE 0.25         //This sets the scale to discretize the energy
#define EF_PROTONS          //The Fermi Energy for PROTONS
#define EF_NEUTRONS         //The Fermi Energy for NEUTRONS

class levelinfoclass{
public:
  int nlevels,amax,antistatecheck;
  int jcode[NLEVELS_ARRAY];
  int ienergy[NLEVELS_ARRAY];
  int antistate[NLEVELS_ARRAY];
  int ie0[AMAX_ARRAY];
};

void getrho(long long rho[AMAX_ARRAY][EMAX_ARRAY][JMAX_ARRAY],int iemax,levelinfoclass *levelinfo,int jjmax[AMAX_ARRAY][EMAX_ARRAY]);

void shell_sort(int n, double *a, int *b);

void levelsetter(char *filename,int iemax,double ef,int upordown,levelinfoclass *levelinfo);

void main(){
  int upordown,ie,iiee,i,ii,j,a,iemax,amax;
  int jmin,jmax,jp,jh,iep,ieh,amaxparts,sign;
  int jjmax_parts[AMAX_ARRAY][EMAX_ARRAY];
  int jjmax_holes[AMAX_ARRAY][EMAX_ARRAY];
  int inp;
  long long rhop[AMAX_ARRAY][EMAX_ARRAY][JMAX_ARRAY];
  long long rhoh[AMAX_ARRAY][EMAX_ARRAY][JMAX_ARRAY];
  double totalrho[EMAX_ARRAY][2*JMAX_ARRAY]={0.0};
  double netrho[EMAX_ARRAY];
  double ef,energy;
  char filename[100];
  char pnchar[2];
  levelinfoclass levelinfo;
  ifstream hflevels_file;
  ofstream results_file,netrho_file;

  iemax=EMAX_ARRAY-1;
  
  for(inp=0;inp<2;inp++){

    for(ie=0;ie<EMAX_ARRAY;ie++){
      for(j=0;j<2*JMAX_ARRAY;j++){
	totalrho[ie][j]=0.0;
      }
    }

    if(inp==0){
      cout << "NEUTRONS\n";
      ef=-15.5;
      pnchar[0]='n';
    }
    if(inp==1){
      cout << "PROTONS\n";
      ef=-6.0;
      pnchar[0]='p';
    }

    // FIRST DO THE PARTICLES
    upordown=1;
    strcpy(filename,form("hartreefock/%clevels.dat\0",pnchar[0]));
    levelsetter(filename,iemax,ef,upordown,&levelinfo);

    amaxparts=levelinfo.amax;
    cout << form("amax=%d  nlevels=%d\n",levelinfo.amax,levelinfo.nlevels);

    getrho(rhop,iemax,&levelinfo,jjmax_parts);
    
    // NOW DO THE HOLES

    upordown=-1;
    strcpy(filename,form("hartreefock/%clevels.dat\0",pnchar[0]));
    levelsetter(filename,iemax,ef,upordown,&levelinfo);

    amax=levelinfo.amax;
    cout << form("amax=%d  nlevels=%d\n",levelinfo.amax,levelinfo.nlevels);

    getrho(rhoh,iemax,&levelinfo,jjmax_holes);


    // CONVOLUTE PARTICLES AND HOLES

    if(amax>amaxparts) amax=amaxparts;

    sign=-1;
    for(a=0;a<=amax;a++){
      cout << form("coupling for a=%d\n",a);
      sign=-sign;
      for(iep=0;iep<=iemax;iep++){
	for(ieh=0;ieh<=iemax-iep;ieh++){
	  ie=iep+ieh;
	  for(jp=0;jp<=jjmax_parts[a][iep];jp++){
	    for(jh=0;jh<=jjmax_holes[a][ieh];jh++){
	      jmin=abs(jp-jh);
	      jmax=jp+jh;
	      if(sign==-1) jmax=jmax+1;
	      if(jmax>=2*JMAX_ARRAY) {
		cout << "DANGER, falling apart in convolution\n";
	      }
	      for(j=jmin;j<=jmax;j++){
		totalrho[ie][j]=totalrho[ie][j]
		  +rhop[a][iep][jp]*rhoh[a][ieh][jh];
	      }
	    }
	  }
	}
      }
    }
  
    // OUTPUT

    for(j=0;j<JMAX_ARRAY;j++){
    
      strcpy(filename,form("%cresults/rho_j%03d.dat\0",pnchar[0],j));
      results_file.open(filename);
      results_file << "   ie   Estar(MeV)    rho(MeV^-1) \n";
      for(ie=0;ie<=iemax;ie++){
	energy=ie*ESCALE;
	results_file << setw(5) << ie << setw(10) << energy << setw(24) << totalrho[ie][j] << setw(24) << totalrho[ie][j]*(2*j+1) << endl;
      }
      results_file.close();
    }

    strcpy(filename,form("netrho_%c.dat\0",pnchar[0]));
    netrho_file.open(filename);
    for(ie=0;ie<=iemax;ie++){
      netrho[ie]=0.0;
      for(j=0;j<=JMAX_ARRAY;j++){
	netrho[ie]=netrho[ie]+(2*j+1)*(double)totalrho[ie][j];
      }
      netrho_file << setw(5) << ie << setw(20) << netrho[ie] << endl;
    }
    netrho_file.close();
  }

}

#include "alexsubs.cpp"
